# Generated by roxygen2: do not edit by hand

S3method(pander,"NULL")
S3method(pander,"function")
S3method(pander,Arima)
S3method(pander,CrossTable)
S3method(pander,Date)
S3method(pander,Glm)
S3method(pander,POSIXct)
S3method(pander,POSIXlt)
S3method(pander,anova)
S3method(pander,aov)
S3method(pander,aovlist)
S3method(pander,call)
S3method(pander,cast_df)
S3method(pander,character)
S3method(pander,clogit)
S3method(pander,coxph)
S3method(pander,cph)
S3method(pander,data.frame)
S3method(pander,data.table)
S3method(pander,default)
S3method(pander,density)
S3method(pander,describe)
S3method(pander,ets)
S3method(pander,evals)
S3method(pander,factor)
S3method(pander,formula)
S3method(pander,ftable)
S3method(pander,glm)
S3method(pander,gtable)
S3method(pander,htest)
S3method(pander,image)
S3method(pander,irts)
S3method(pander,list)
S3method(pander,lm)
S3method(pander,lme)
S3method(pander,logical)
S3method(pander,lrm)
S3method(pander,manova)
S3method(pander,matrix)
S3method(pander,microbenchmark)
S3method(pander,name)
S3method(pander,nls)
S3method(pander,numeric)
S3method(pander,ols)
S3method(pander,orm)
S3method(pander,polr)
S3method(pander,prcomp)
S3method(pander,randomForest)
S3method(pander,rapport)
S3method(pander,rlm)
S3method(pander,sessionInfo)
S3method(pander,smooth.spline)
S3method(pander,stat.table)
S3method(pander,summary.aov)
S3method(pander,summary.aovlist)
S3method(pander,summary.glm)
S3method(pander,summary.lm)
S3method(pander,summary.lme)
S3method(pander,summary.manova)
S3method(pander,summary.nls)
S3method(pander,summary.polr)
S3method(pander,summary.prcomp)
S3method(pander,summary.rms)
S3method(pander,summary.survreg)
S3method(pander,summary.table)
S3method(pander,survdiff)
S3method(pander,survfit)
S3method(pander,survreg)
S3method(pander,table)
S3method(pander,tabular)
S3method(pander,ts)
S3method(pander,vector)
S3method(pander,zoo)
export(Pandoc)
export(Pandoc.brew)
export(Pandoc.convert)
export(add.blank.lines)
export(add.significance.stars)
export(cache.off)
export(cache.on)
export(emphasize.cells)
export(emphasize.cols)
export(emphasize.italics.cells)
export(emphasize.italics.cols)
export(emphasize.italics.rows)
export(emphasize.rows)
export(emphasize.strong.cells)
export(emphasize.strong.cols)
export(emphasize.strong.rows)
export(emphasize.verbatim.cells)
export(emphasize.verbatim.cols)
export(emphasize.verbatim.rows)
export(eval.msgs)
export(evals)
export(evalsOptions)
export(has.rownames)
export(openFileInOS)
export(p)
export(pander)
export(panderOptions)
export(pander_return)
export(pandoc.date)
export(pandoc.date.return)
export(pandoc.emphasis)
export(pandoc.emphasis.return)
export(pandoc.footnote)
export(pandoc.footnote.return)
export(pandoc.formula)
export(pandoc.formula.return)
export(pandoc.header)
export(pandoc.header.return)
export(pandoc.horizontal.rule)
export(pandoc.horizontal.rule.return)
export(pandoc.image)
export(pandoc.image.return)
export(pandoc.indent)
export(pandoc.link)
export(pandoc.link.return)
export(pandoc.list)
export(pandoc.list.return)
export(pandoc.p)
export(pandoc.p.return)
export(pandoc.strikeout)
export(pandoc.strikeout.return)
export(pandoc.strong)
export(pandoc.strong.return)
export(pandoc.table)
export(pandoc.table.return)
export(pandoc.title)
export(pandoc.title.return)
export(pandoc.verbatim)
export(pandoc.verbatim.return)
export(path_to_pandoc)
export(redraw.recordedplot)
export(redrawPlot)
export(remove.extra.newlines)
export(repChar)
export(set.alignment)
export(set.caption)
export(splitLine)
export(trim.spaces)
export(wrap)
importFrom(Rcpp,evalCpp)
importFrom(digest,digest)
importFrom(grDevices,dev.control)
importFrom(grDevices,dev.list)
importFrom(grDevices,dev.off)
importFrom(grDevices,recordPlot)
importFrom(grDevices,replayPlot)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,par)
importFrom(methods,getGeneric)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setRefClass)
importFrom(stats,coef)
importFrom(stats,cycle)
importFrom(stats,end)
importFrom(stats,frequency)
importFrom(stats,ftable)
importFrom(stats,naprint)
importFrom(stats,pchisq)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,start)
importFrom(stats,time)
importFrom(stats,vcov)
importFrom(tools,file_path_sans_ext)
importFrom(utils,as.roman)
importFrom(utils,capture.output)
importFrom(utils,getFromNamespace)
importFrom(utils,head)
importFrom(utils,object.size)
importFrom(utils,packageDescription)
importFrom(utils,packageVersion)
importFrom(utils,tail)
useDynLib(pander)
