% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{get.caption}
\alias{get.caption}
\title{Get caption}
\usage{
get.caption()
}
\value{
stored caption as string
}
\description{
Get caption from temporary environment and truncates that
}
\keyword{internal}
