% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{get.storage}
\alias{get.storage}
\title{Get a value from internal buffer}
\usage{
get.storage(what)
}
\arguments{
\item{what}{string}
}
\description{
And truncate content.
}
\keyword{internal}
